/*
    ircprotocol.h - IRC Protocol

    Copyright (c) 2002      by Nick Betcher <nbetcher@kde.org>

    Kopete    (c) 2002      by the Kopete developers <kopete-devel@kde.org>

    *************************************************************************
    *                                                                       *
    * This program is free software; you can redistribute it and/or modify  *
    * it under the terms of the GNU General Public License as published by  *
    * the Free Software Foundation; either version 2 of the License, or     *
    * (at your option) any later version.                                   *
    *                                                                       *
    *************************************************************************
*/

#ifndef IRCPROTOCOL_H
#define IRCPROTOCOL_H

#include "kopeteonlinestatus.h"
#include "kopeteprotocol.h"

class KopeteMetaContact;
class AddContactPage;
class KIRC;

class EditAccountWidget;
class KopeteAccount;
class IRCAccount;
class KActionCollection;

class QStringList;
class QWidget;
class KSParser;

/**
 * @author Nick Betcher <nbetcher@kde.org>
 */
class IRCProtocol : public KopeteProtocol
{
	Q_OBJECT

public:
	IRCProtocol( QObject *parent, const char *name, const QStringList &args );
	~IRCProtocol();

	/** KopeteProtocol reimplementation */
	virtual AddContactPage *createAddContactWidget(QWidget *parent, KopeteAccount *account);

	/**
	 * Deserialize contact data
	 */
	virtual void deserializeContact( KopeteMetaContact *metaContact,
		const QMap<QString, QString> &serializedData, const QMap<QString, QString> &addressBookData );

	virtual EditAccountWidget* createEditAccountWidget(KopeteAccount *account, QWidget *parent);

	virtual KopeteAccount* createNewAccount(const QString &accountId);

	virtual KActionCollection *customChatWindowPopupActions( const KopeteMessage &, DOM::Node & );

	/**
	 * Returns a pointer to the KSParser. The KSParser changes IRC color codes into HTML
	 * for use in KopeteMessage.
	 */
	KSParser *parser() const { return mParser; };

	static IRCProtocol *protocol();

	static KopeteOnlineStatus IRCChannelOnline() { return m_ChannelOnline; };
	static KopeteOnlineStatus IRCUserConnecting() { return m_UserConnecting; };
	static KopeteOnlineStatus IRCUserOnline() { return m_UserOnline; };
	static KopeteOnlineStatus IRCChannelOffline() { return m_ChannelOffline; };
	static KopeteOnlineStatus IRCUserOffline() { return m_UserOffline; };
	static KopeteOnlineStatus IRCUserOp() { return m_UserOp; };
	static KopeteOnlineStatus IRCUserVoice() { return m_UserVoice; };
	static KopeteOnlineStatus IRCUserAway() { return m_UserAway; };

private slots:
	void slotMessageFilter( KopeteMessage &msg );

	void slotTopicCommand( const QString &args, KopeteMessageManager *manager);
	void slotJoinCommand( const QString &args, KopeteMessageManager *manager);
	void slotNickCommand( const QString &args, KopeteMessageManager *manager);
	void slotWhoisCommand( const QString &args, KopeteMessageManager *manager);
	void slotMeCommand( const QString &args, KopeteMessageManager *manager);
	void slotModeCommand( const QString &args, KopeteMessageManager *manager);
	void slotQueryCommand( const QString &args, KopeteMessageManager *manager);

	void slotKickCommand( const QString &args, KopeteMessageManager *manager);
	void slotBanCommand( const QString &args, KopeteMessageManager *manager);
	void slotOpCommand( const QString &args, KopeteMessageManager *manager);
	void slotDeopCommand( const QString &args, KopeteMessageManager *manager);
	void slotVoiceCommand( const QString &args, KopeteMessageManager *manager);
	void slotDevoiceCommand( const QString &args, KopeteMessageManager *manager);


private:
	/** FIXME: Do something with this when Account support is added!!!!!!!! */
	IRCAccount *account;

	static IRCProtocol *s_protocol;

	static KopeteOnlineStatus m_ChannelOnline;
	static KopeteOnlineStatus m_ChannelOffline;
	static KopeteOnlineStatus m_UserOnline;
	static KopeteOnlineStatus m_UserOp;
	static KopeteOnlineStatus m_UserVoice;
	static KopeteOnlineStatus m_UserOffline;
	static KopeteOnlineStatus m_UserAway;
	static KopeteOnlineStatus m_UserConnecting;
	static KopeteOnlineStatus m_Unknown;

	KActionCollection *mActions;

	void simpleModeChange( const QString &, KopeteMessageManager *, const QString &mode );

	KSParser *mParser;
	DOM::Node activeNode;
	IRCAccount *activeAccount;
};

#endif

// vim: set noet ts=4 sts=4 sw=4:

